/*******************************************************************************
 *                                                                             *
 *   Author: Victor Mateevitsi (mvictoras@gmail.com)                           *
 *                                                                             *
 *   This file is part of CS 491 Project1.                                     *
 *                                                                             * 
 *   CS 491 Project1 is free software: you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by      *
 *   the Free Software Foundation, either version 3 of the License, or         *
 *   (at your option) any later version.                                       *
 *                                                                             *
 *   CS 491 Project1 is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *   GNU General Public License for more details.                              *
 *                                                                             *
 *   You should have received a copy of the GNU General Public License         *
 *   along with CS 491 Project1.  If not, see <http://www.gnu.org/licenses/>.  *
 *                                                                             *
 *******************************************************************************/

/********************************************************************************
 *                           class TimeLineChart                                *
 *                    Chart - > LineChart -> TimeLineChart                      *
 * TimeLineChart class is class for creating timeline line charts. The main     *
 * difference with the line chart, is that the X axis is composed of dates      *
 * which are of type String and not float.                                      *
 *                                                                              *
 ********************************************************************************/
 
class TimeLineChart extends Chart {
  
 /********************************************************************************
  *                                                                              *
  *                               Class Members                                  *
  *                                                                              *
  ********************************************************************************/
  
  private String[] xDateValues; // The dates of the plotted data.
  private boolean isBarChart;   // If this is true, a bar chart instead of a line chart is plotted.
  
 /********************************************************************************
  *                                                                              *
  *                               Constructors                                   *
  *                                                                              *
  ********************************************************************************/
  
  TimeLineChart(int chartWidth, int chartHeight, String labelX, String labelY, String title, float[] xValues, float[] yValues) {
    super(chartWidth, chartHeight, labelX, labelY, title, xValues, yValues);
  }
  
  // XXX - Have to rewrite this
  TimeLineChart(int chartWidth, int chartHeight, String labelX, String labelY, String title, String[] xValues, float[] yValues) {
    super(chartWidth, chartHeight, labelX, labelY, title, xValues, yValues);
    this.chartWidth = chartWidth;
    this.chartHeight = chartHeight;
    
    this.labelX = labelX;
    this.labelY = labelY;
    this.title = title;
    
    hasYAxis = true;
    hasXAxis = true;
    hasYLabel = true;
    hasXLabel = true;
    hasTitle = true;
    hasXAxisLines = false;
    hasYAxisLines = true;
    
    this.xDateValues = xValues;
    
    this.yValues = yValues;
    
  //  yMinValue = min(yValues);
  //  yMaxValue = max(yValues);
    
    dataStrokeWeight = DEFAULT_DATASTROKEWEIGHT;
    dataStrokeColor = DEFAULT_DATASTROKECOLOR;
    
    chartLabelsFont = SM_FONT;
    textFont(chartLabelsFont);
    
    isBarChart = false;
  }
  
  
 /********************************************************************************
  *                                                                              * 
  *                               Functions                                      *
  *                                                                              *
  ********************************************************************************/
  
 /*
  * function: drawAxis()
  *
  * Overridden function
  * Draws the X axis of the Chart with the appropiate labels
  *
  */
  void drawXAxis() 
  {
    stroke(0);
    fill(0);
    strokeWeight(1);
    textSize(8);
    textAlign(CENTER, CENTER);
    line(0, chartHeight, chartWidth, chartHeight);
    
    for( int i = 0; i < xDateValues.length; i++ ) 
    {
      if( i % xInterval == 0 ) 
      {
        float xCoord = map(i, 0, xDateValues.length, 5, chartWidth);
        line(xCoord, chartHeight, xCoord, chartHeight + 5);
        
        text(xDateValues[i], xCoord, chartHeight + 10);
      }
    }
  }
  
 /*
  * function: drawXAxisLines()
  *
  * Overridden function 
  * Draws lines parallel to the X axis
  */
  void drawXAxisLines() {
    stroke(224);
    strokeWeight(0.5);
    
    for(int i = 1; i < xDateValues.length - 1; i++) {
      float xLabelCoord = map(i, 0, xDateValues.length, 5, chartWidth);
      line(xLabelCoord, 0, xLabelCoord, chartHeight - 1);
    }
  }
  
 /*
  * function: drawData()
  * 
  * Overridden function.
  * Draws the actual data of the plotted chart.
  */
  void drawData() {
    drawData(this.yValues, dataStrokeColor);
  }
  
 /*
  * function: drawData(float[], int)
  * 
  * xValues    : The array with the values of the X axis to be plotted
  * chartColor : The color of the plotted data
  *
  * Overloaded function.
  * Draws the actual data of the plotted chart. 
  */
  void drawData(float[] yValues, int lineColor) {
    float xCoord, yCoord;
    rectMode(CORNERS);
  
    noFill();
    stroke(lineColor);
    strokeWeight(dataStrokeWeight);
      
    beginShape();
      for(int i = 0; i < xDateValues.length; i++) {  
        xCoord = map(i, 0, xDateValues.length, 5, chartWidth);
        yCoord = map(yValues[i], yMinValue, yMaxValue, chartHeight, 0);
        vertex(xCoord, yCoord);
      }
    endShape();
  }
}

